!{\src2tex{textfont=tt}}
!!****f* ABINIT/defs_mach
!! NAME
!! defs_mach
!!
!! FUNCTION
!! This module contains definitions for a number of
!! machine dependent constants
!!
!! COPYRIGHT
!! Copyright (C) 1998-2005 ABINIT group (NH, FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!!
!! NOTES
!! Comes from the module GlobalConstants
!! of the code atompaw by N.A.W. Holzwarth
!!
!! SOURCE

 module defs_mach

 use defs_basis

 implicit none

!Real constants
 real(dp) :: machine_precision,machine_zero,machine_infinity
 real(dp) :: minlog,maxlog,minexp,maxexp
 real(dp) :: minlogarg,maxlogarg,minexparg,maxexparg

 contains
  Subroutine Init_defs_mach
  implicit none
  integer :: i
  real(dp) :: tmp, a1,a2,a3
  machine_precision = 0
  a1 = 4.d0/3.d0
  do while (machine_precision == 0.d0)
   a2 = a1 - 1.d0
   a3 = a2 + a2 + a2
   machine_precision = ABS(a3 - 1.d0)
  enddo
  machine_zero= machine_precision**4
  machine_infinity = 1.d0/machine_zero
  minlogarg=machine_precision; minlog=log(minlogarg)
  maxlogarg=1.d0/machine_precision; maxlog=log(maxlogarg)
  minexparg=log(machine_precision);  minexp=0.d0
  maxexparg=-log(machine_precision);  maxexp=exp(maxexparg)
  end subroutine

  !****************************************************
  function ddlog(arg)
    real(dp) :: arg,ddlog

    IF (arg>maxlogarg) THEN
       ddlog=maxlog
    ELSE IF (arg<minlogarg) THEN
       ddlog=minlog
    ELSE
       ddlog=LOG(arg)
    ENDIF

    RETURN
  END FUNCTION ddlog

  !****************************************************
  Function ddexp(arg)
    real(dp) :: arg,ddexp

    IF (arg>maxexparg) THEN
       ddexp=maxexp
    ELSE IF (arg<minexparg) THEN
       ddexp=minexp
    ELSE
       ddexp=EXP(arg)
    ENDIF
  END FUNCTION ddexp

  !****************************************************
  !Function used to interface "csimp" routine with "overint" function from AtomPAW
  function overint(n,h,f1)
   use defs_basis
   real(dp) :: overint
   integer :: n
   real(dp) :: h,f1(n)
   call csimp(f1,n,h,overint)
  end function overint

 end module defs_mach
!!***
